# Debian
|Python Version|Debian 11 (bullseye)|Debian 12 (bookworm)|
|:------------:|:------------------:|:------------------:|
|     3.9      |         X          |                    |
|     3.11     |                    |         X          |

[Debian](https://www.debian.org/) is a general-purpose operating system often used as a base for other operating systems, notably Ubuntu.

Debian "Stable" is released every two years, and notably, only security updates are allowed in this time.
This helps makes Debian an exceedinly reliable foundation, at the cost of no new software features for two years at a time.

Debian is usually used with the Linux kernel; thus, "Debian Linux".

## Installing Python on Debian Linux
Python often comes pre-installed on Debian.
Sometimes it doesn't, for example on rented cloud servers.

You can always install Python securely and easily the usual way with apt. **Note: Remember to pre-append apt with `sudo`**
```bash
# image: docker.io/debian:bookworm
# command: bash
# user: root
apt update
apt install python3

# Remember, `sudo ...` lets any "administrator" user run one command as root.
# - Ex. `sudo apt update`.
```

Make sure to check that `python` is available:
```bash
# user: any
python3 --version
>>> Python 3.11.2

python3 -c "print('Hello World')"
>>> Hello World!
```

For convenience, we suggest you also install `python-is-python3`
```bash
# user: root
apt install python-is-python3
```
which lets you invoke Python using `python` instead of `python3`:
```bash
# user: any
python --version
>>> Python 3.10.12
```



# Important Information
Here are some things to keep in mind when using Python on Debian.

## Don't Use pip (without venv)
Many websites will encourage you to run things like `sudo pip install`.

**Don't**.

**Seriously, [Don't Break Debian](https://wiki.debian.org/DontBreakDebian)**.

If you need packages on your Debian system, **always** prefer the built-in Debian packages. Here are some that are commonly used at DTU:
- `numpy`: <https://packages.debian.org/bookworm/python3-numpy>
- `pandas`: <https://packages.debian.org/bookworm/python3-pandas>
- `scipy`: <https://packages.debian.org/bookworm/python3-scipy>
- `matplotlib`: <https://packages.debian.org/bookworm/python3-matplotlib>
- `seaborn`: <https://packages.debian.org/bookworm/python3-seaborn>

You can find such packages on [Debian's package search](https://packages.debian.org/search?keywords=search).

### ...But I need the newest packages / a specific version of a package!
This is what `venv`s ("Virtual Environments") are for.
See [our article on venv](../packages/venv.md).


## What's with the Toy Story release names?
In 1995, the Debian Project Leader worked at Pixar.
Thus, Debian was granted permission to name releases after Toy Story characters.

The tradition continues to this day!



# Reference
- Debian Wiki on Python: <https://wiki.debian.org/Python>
- Bugs in Debian Python: <https://bugs.debian.org/python3>
- Python Package Info: <https://packages.debian.org/bookworm/python3>
- Debian Package Tracker: <https://tracker.debian.org/pkg/python3-defaults>
- Debian Wiki on Toy Story: <https://wiki.debian.org/ToyStory>
