# Ubuntu
|Python Version|Ubuntu 20.04 (focal)|Ubuntu 22.04 (jammy)|
|:------------:|:------------------:|:------------------:|
|     3.8      |         ~          |                    |
|     3.10     |                    |         X          |

[Ubuntu Linux](https://ubuntu.com/desktop) is a general-purpose Debian-based operating system, developed and commercially supported by Canonical Ltd.

Ubuntu LTS ("Long Term Support") is released every two years, and notably, only security updates are allowed in LTS during this time.
Smaller, optional releases between LTS releases provide newer software to users who don't need maximum stability.

Ubuntu is known especially for its:
- Beginner-friendly desktop focus on making any computer "just work", including official laptops with pre-installed Ubuntu from ex. Dell.
- Wide range of [flavors](https://ubuntu.com/desktop/flavours) with their own take on modern, productive interfaces.
- Availability of full-stack commercial support for server flavors. 

## Installing Python on Ubuntu Linux
Python often comes pre-installed on Ubuntu.
Sometimes it doesn't, for example on rented cloud servers.

You can always install Python securely and easily the usual way:
```bash
# image: docker.io/ubuntu:jammy
# command: bash
# user: root
apt update
apt install python3

# Remember, `sudo ...` lets any "administrator" user run one command as root.
# - Ex. `sudo apt update`.
```

Make sure to check that `python` is available:
```bash
# user: any
python3 --version
>>> Python 3.10.12

python3 -c "print('Hello World')"
>>> Hello World!
```

For convenience, we suggest you also install `python-is-python3`
```bash
# user: root
apt install python-is-python3
```
which lets you invoke Python using `python` instead of `python3`:
```bash
# user: any
python --version
>>> Python 3.10.12
```



# Important Information
Here are some things to keep in mind when using Python on Ubuntu.

## Don't Use pip (without venv)
Many websites will encourage you to run things like `sudo pip install`.

**Don't**.

**Seriously, [Don't Break Debian (/Ubuntu)](https://wiki.debian.org/DontBreakDebian)**.

If you need packages on your Ubuntu system, **always** prefer the built-in Ubuntu packages. Here are some that are commonly used at DTU:
- `numpy`: <https://packages.ubuntu.com/jammy/python3-numpy>
- `pandas`: <https://packages.ubuntu.com/jammy/python3-pandas>
- `scipy`: <https://packages.ubuntu.com/jammy/python3-scipy>
- `matplotlib`: <https://packages.ubuntu.com/jammy/python3-matplotlib>
- `seaborn`: <https://packages.ubuntu.com/jammy/python3-seaborn>

You can find such packages on [Ubuntu's package search](https://packages.ubuntu.com/).

### ...But I need the newest packages / a specific version of a package!
This is what `venv`s ("Virtual Environments") are for.
See [our article on venv](../packages/venv.md).



# Reference
- Ubuntu Wiki on Python: <https://wiki.ubuntu.com/Python>
- Bugs in Ubuntu Python: <https://launchpad.net/ubuntu/+source/python3-defaults/+bugs>
- Python Package Info: <https://packages.ubuntu.com/jammy/python3>
